@@ -8,7 +8,7 @@ package Excel::Writer::XLSX::Chart::Pie;
 #
 # See formatting note in Excel::Writer::XLSX::Chart.
 #
-# Copyright 2000-2012, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2013, John McNamara, jmcnamara@cpan.org
 #
 # Documentation after __END__
 #
@@ -22,7 +22,7 @@ use Carp;
 use Excel::Writer::XLSX::Chart;
 
 our @ISA     = qw(Excel::Writer::XLSX::Chart);
-our $VERSION = '0.65';
+our $VERSION = '0.76';
 
 
 ###############################################################################
@@ -99,7 +99,7 @@ sub _write_plot_area {
     $self->xml_start_tag( 'c:plotArea' );
 
     # Write the c:layout element.
-    $self->_write_layout();
+    $self->_write_layout( $self->{_plotarea}->{_layout}, 'plot' );
 
     # Write the subclass chart type element.
     $self->_write_chart_type();
@@ -118,9 +118,17 @@ sub _write_plot_area {
 #
 sub _write_legend {
 
-    my $self     = shift;
-    my $position = $self->{_legend_position};
-    my $overlay  = 0;
+    my $self          = shift;
+    my $position      = $self->{_legend_position};
+    my $font          = $self->{_legend_font};
+    my @delete_series = ();
+    my $overlay       = 0;
+
+    if ( defined $self->{_legend_delete_series}
+        && ref $self->{_legend_delete_series} eq 'ARRAY' )
+    {
+        @delete_series = @{ $self->{_legend_delete_series} };
+    }
 
     if ( $position =~ s/^overlay_// ) {
         $overlay = 1;
@@ -143,14 +151,21 @@ sub _write_legend {
     # Write the c:legendPos element.
     $self->_write_legend_pos( $position );
 
+    # Remove series labels from the legend.
+    for my $index ( @delete_series ) {
+
+        # Write the c:legendEntry element.
+        $self->_write_legend_entry( $index );
+    }
+
     # Write the c:layout element.
-    $self->_write_layout();
+    $self->_write_layout( $self->{_legend_layout}, 'legend' );
 
     # Write the c:overlay element.
     $self->_write_overlay() if $overlay;
 
     # Write the c:txPr element. Over-ridden.
-    $self->_write_tx_pr_legend();
+    $self->_write_tx_pr_legend( 0, $font );
 
     $self->xml_end_tag( 'c:legend' );
 }
@@ -164,19 +179,25 @@ sub _write_legend {
 #
 sub _write_tx_pr_legend {
 
-    my $self  = shift;
-    my $horiz = 0;
+    my $self     = shift;
+    my $horiz    = shift;
+    my $font     = shift;
+    my $rotation = undef;
+
+    if ( $font && exists $font->{_rotation} ) {
+        $rotation = $font->{_rotation};
+    }
 
     $self->xml_start_tag( 'c:txPr' );
 
     # Write the a:bodyPr element.
-    $self->_write_a_body_pr( $horiz );
+    $self->_write_a_body_pr( $rotation, $horiz );
 
     # Write the a:lstStyle element.
     $self->_write_a_lst_style();
 
     # Write the a:p element.
-    $self->_write_a_p_legend();
+    $self->_write_a_p_legend( $font );
 
     $self->xml_end_tag( 'c:txPr' );
 }
@@ -190,13 +211,13 @@ sub _write_tx_pr_legend {
 #
 sub _write_a_p_legend {
 
-    my $self  = shift;
-    my $title = shift;
+    my $self = shift;
+    my $font = shift;
 
     $self->xml_start_tag( 'a:p' );
 
     # Write the a:pPr element.
-    $self->_write_a_p_pr_legend();
+    $self->_write_a_p_pr_legend( $font );
 
     # Write the a:endParaRPr element.
     $self->_write_a_end_para_rpr();
@@ -214,6 +235,7 @@ sub _write_a_p_legend {
 sub _write_a_p_pr_legend {
 
     my $self = shift;
+    my $font = shift;
     my $rtl  = 0;
 
     my @attributes = ( 'rtl' => $rtl );
@@ -221,7 +243,7 @@ sub _write_a_p_pr_legend {
     $self->xml_start_tag( 'a:pPr', @attributes );
 
     # Write the a:defRPr element.
-    $self->_write_a_def_rpr();
+    $self->_write_a_def_rpr( $font );
 
     $self->xml_end_tag( 'a:pPr' );
 }
@@ -412,6 +434,6 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright MM-MMXII, John McNamara.
+Copyright MM-MMXIIII, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
