@@ -6,7 +6,7 @@ package Excel::Writer::XLSX::Package::Packager;
 #
 # Used in conjunction with Excel::Writer::XLSX
 #
-# Copyright 2000-2012, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2013, John McNamara, jmcnamara@cpan.org
 #
 # Documentation after __END__
 #
@@ -31,7 +31,7 @@ use Excel::Writer::XLSX::Package::Theme;
 use Excel::Writer::XLSX::Package::VML;
 
 our @ISA     = qw(Exporter);
-our $VERSION = '0.65';
+our $VERSION = '0.76';
 
 
 ###############################################################################
@@ -763,8 +763,12 @@ sub _write_drawing_rels_files {
 
     my $index = 0;
     for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+
+        if ( @{ $worksheet->{_drawing_links} } || $worksheet->{_has_shapes} ) {
+            $index++;
+        }
+
         next unless @{ $worksheet->{_drawing_links} };
-        $index++;
 
         # Create the drawing .rels dir.
         _mkdir( $dir . '/xl' );
@@ -904,7 +908,7 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
-(c) MM-MMXII, John McNamara.
+(c) MM-MMXIIII, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
 
