@@ -7,7 +7,7 @@ package Excel::Writer::XLSX::Utility;
 #
 # Used in conjunction with Excel::Writer::XLSX
 #
-# Copyright 2000-2012, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2013, John McNamara, jmcnamara@cpan.org
 #
 # Documentation after __END__
 #
@@ -21,7 +21,7 @@ use warnings;
 use autouse 'Date::Calc'  => qw(Delta_DHMS Decode_Date_EU Decode_Date_US);
 use autouse 'Date::Manip' => qw(ParseDate Date_Init);
 
-our $VERSION = '0.46';
+our $VERSION = '0.76';
 
 # Row and column functions
 my @rowcol = qw(
@@ -87,6 +87,8 @@ sub xl_cell_to_rowcol {
 
     my $cell = shift;
 
+    return ( 0, 0, 0, 0 ) unless $cell;
+
     $cell =~ /(\$?)([A-Z]{1,3})(\$?)(\d+)/;
 
     my $col_abs = $1 eq "" ? 0 : 1;
@@ -283,12 +285,12 @@ sub xl_parse_time {
 
     my $time = shift;
 
-    if ( $time =~ /(\d{1,2}):(\d\d):?((?:\d\d)(?:\.\d+)?)?(?:\s+)?(am|pm)?/i ) {
+    if ( $time =~ /(\d+):(\d\d):?((?:\d\d)(?:\.\d+)?)?(?:\s+)?(am|pm)?/i ) {
 
         my $hours    = $1;
         my $minutes  = $2;
         my $seconds  = $3 || 0;
-        my $meridian = lc( $4 ) || '';
+        my $meridian = lc( $4 || '' );
 
         # Normalise midnight and midday
         $hours = 0 if ( $hours == 12 && $meridian ne '' );
@@ -314,8 +316,6 @@ sub xl_parse_date {
 
     my $date = ParseDate( $_[0] );
 
-    return undef unless defined $date;
-
     # Unpack the return value from ParseDate()
     my ( $years, $months, $days, $hours, undef, $minutes, undef, $seconds ) =
       unpack( "A4     A2      A2     A2      C        A2      C       A2",
@@ -349,9 +349,8 @@ sub xl_decode_date_EU {
     my $time = 0;
 
     # Remove and decode the time portion of the string
-    if ( $date =~ s/(\d{1,2}:\d\d:?(\d\d(\.\d+)?)?(\s+)?(am|pm)?)//i ) {
+    if ( $date =~ s/(\d+:\d\d:?(\d\d(\.\d+)?)?(\s+)?(am|pm)?)//i ) {
         $time = xl_parse_time( $1 );
-        return undef unless defined $time;
     }
 
     # Return if the string is now blank, i.e. it contained a time only.
@@ -378,9 +377,8 @@ sub xl_decode_date_US {
     my $time = 0;
 
     # Remove and decode the time portion of the string
-    if ( $date =~ s/(\d{1,2}:\d\d:?(\d\d(\.\d+)?)?(\s+)?(am|pm)?)//i ) {
+    if ( $date =~ s/(\d+:\d\d:?(\d\d(\.\d+)?)?(\s+)?(am|pm)?)//i ) {
         $time = xl_parse_time( $1 );
-        return undef unless defined $time;
     }
 
     # Return if the string is now blank, i.e. it contained a time only.
@@ -430,18 +428,18 @@ Functions to help with some common tasks when using L<Excel::Writer::XLSX>.
 
 These functions mainly relate to dealing with rows and columns in A1 notation and to handling dates and times.
 
-    use Excel::Writer::XLSX::Utility;                   # Import everything
+    use Excel::Writer::XLSX::Utility;                     # Import everything
 
-    ($row, $col)    = xl_cell_to_rowcol('C2');          # (1, 2)
-    $str            = xl_rowcol_to_cell(1, 2);          # C2
-    $str            = xl_col_to_name(702);              # AAA
-    $str            = xl_inc_col('Z1'  );               # AA1
-    $str            = xl_dec_col('AA1' );               # Z1
+    ($row, $col)    = xl_cell_to_rowcol( 'C2' );          # (1, 2)
+    $str            = xl_rowcol_to_cell( 1, 2 );          # C2
+    $str            = xl_col_to_name( 702 );              # AAA
+    $str            = xl_inc_col( 'Z1'  );                # AA1
+    $str            = xl_dec_col( 'AA1' );                # Z1
 
-    $date           = xl_date_list(2002, 1, 1);         # 37257
-    $date           = xl_parse_date("11 July 1997");    # 35622
-    $time           = xl_parse_time('3:21:36 PM');      # 0.64
-    $date           = xl_decode_date_EU("13 May 2002"); # 37389
+    $date           = xl_date_list(2002, 1, 1);           # 37257
+    $date           = xl_parse_date( '11 July 1997' );    # 35622
+    $time           = xl_parse_time( '3:21:36 PM' );      # 0.64
+    $date           = xl_decode_date_EU( '13 May 2002' ); # 37389
 
 =head1 DESCRIPTION
 
@@ -480,7 +478,7 @@ All of these functions are exported by default. However, you can use import list
 
 L<Excel::Writer::XLSX> supports two forms of notation to designate the position of cells: Row-column notation and A1 notation.
 
-Row-column notation uses a zero based index for both row and column while A1 notation uses the standard Excel alphanumeric sequence of column letter and 1-based row. Columns range from A to IV i.e. 0 to 255, rows range from 1 to 16384 in Excel 5 and 65536 in Excel 97. For example:
+Row-column notation uses a zero based index for both row and column while A1 notation uses the standard Excel alphanumeric sequence of column letter and 1-based row. Columns range from A to XFD, i.e. 0 to 16,383, rows range from 0 to 1,048,575 in Excel 2007+. For example:
 
     (0, 0)      # The top left cell in row-column notation.
     ('A1')      # The top left cell in A1 notation.
@@ -490,19 +488,19 @@ Row-column notation uses a zero based index for both row and column while A1 not
 
 Row-column notation is useful if you are referring to cells programmatically:
 
-    for my $i (0 .. 9) {
-        $worksheet->write($i, 0, 'Hello'); # Cells A1 to A10
+    for my $i ( 0 .. 9 ) {
+        $worksheet->write( $i, 0, 'Hello' );    # Cells A1 to A10
     }
 
 A1 notation is useful for setting up a worksheet manually and for working with formulas:
 
-    $worksheet->write('H1', 200);
-    $worksheet->write('H2', '=H7+1');
+    $worksheet->write( 'H1', 200 );
+    $worksheet->write( 'H2', '=H7+1' );
 
 The functions in the following sections can be used for dealing with A1 notation, for example:
 
-    ($row, $col)    = xl_cell_to_rowcol('C2');  # (1, 2)
-    $str            = xl_rowcol_to_cell(1, 2);  # C2
+    ( $row, $col ) = xl_cell_to_rowcol('C2');    # (1, 2)
+    $str           = xl_rowcol_to_cell( 1, 2 );  # C2
 
 
 Cell references in Excel can be either relative or absolute. Absolute references are prefixed by the dollar symbol as shown below:
@@ -512,7 +510,7 @@ Cell references in Excel can be either relative or absolute. Absolute references
     A$1     # Column is relative and row is absolute
     $A$1    # Column and row are absolute
 
-An absolute reference only has an effect if the cell is copied. Refer to the Excel documentation for further details. All of the following functions support absolute references.
+An absolute reference only makes a difference if the cell is copied. Refer to the Excel documentation for further details. All of the following functions support absolute references.
 
 =head2 xl_rowcol_to_cell($row, $col, $row_absolute, $col_absolute)
 
@@ -526,18 +524,18 @@ An absolute reference only has an effect if the cell is copied. Refer to the Exc
 
 This function converts a zero based row and column cell reference to a A1 style string:
 
-    $str = xl_rowcol_to_cell(0, 0); # A1
-    $str = xl_rowcol_to_cell(0, 1); # B1
-    $str = xl_rowcol_to_cell(1, 0); # A2
+    $str = xl_rowcol_to_cell( 0, 0 );    # A1
+    $str = xl_rowcol_to_cell( 0, 1 );    # B1
+    $str = xl_rowcol_to_cell( 1, 0 );    # A2
 
 
 The optional parameters C<$row_absolute> and C<$col_absolute> can be used to indicate if the row or column is absolute:
 
-    $str = xl_rowcol_to_cell(0, 0, 0, 1); # $A1
-    $str = xl_rowcol_to_cell(0, 0, 1, 0); # A$1
-    $str = xl_rowcol_to_cell(0, 0, 1, 1); # $A$1
+    $str = xl_rowcol_to_cell( 0, 0, 0, 1 );    # $A1
+    $str = xl_rowcol_to_cell( 0, 0, 1, 0 );    # A$1
+    $str = xl_rowcol_to_cell( 0, 0, 1, 1 );    # $A$1
 
-See L<ROW AND COLUMN FUNCTIONS> for an explanation of absolute cell references.
+See above for an explanation of absolute cell references.
 
 =head2 xl_cell_to_rowcol($string)
 
@@ -548,12 +546,12 @@ See L<ROW AND COLUMN FUNCTIONS> for an explanation of absolute cell references.
 
 This function converts an Excel cell reference in A1 notation to a zero based row and column. The function will also handle Excel's absolute, C<$>, cell notation.
 
-    my ($row, $col) = xl_cell_to_rowcol('A1');     # (0, 0)
-    my ($row, $col) = xl_cell_to_rowcol('B1');     # (0, 1)
-    my ($row, $col) = xl_cell_to_rowcol('C2');     # (1, 2)
-    my ($row, $col) = xl_cell_to_rowcol('$C2' );   # (1, 2)
-    my ($row, $col) = xl_cell_to_rowcol('C$2' );   # (1, 2)
-    my ($row, $col) = xl_cell_to_rowcol('$C$2');   # (1, 2)
+    my ( $row, $col ) = xl_cell_to_rowcol('A1');      # (0, 0)
+    my ( $row, $col ) = xl_cell_to_rowcol('B1');      # (0, 1)
+    my ( $row, $col ) = xl_cell_to_rowcol('C2');      # (1, 2)
+    my ( $row, $col ) = xl_cell_to_rowcol('$C2');     # (1, 2)
+    my ( $row, $col ) = xl_cell_to_rowcol('C$2');     # (1, 2)
+    my ( $row, $col ) = xl_cell_to_rowcol('$C$2');    # (1, 2)
 
 =head2 xl_col_to_name($col, $col_absolute)
 
@@ -565,16 +563,16 @@ This function converts an Excel cell reference in A1 notation to a zero based ro
 
 This function converts a zero based column reference to a string:
 
-    $str = xl_col_to_name(0);   # A
-    $str = xl_col_to_name(1);   # B
-    $str = xl_col_to_name(702); # AAA
+    $str = xl_col_to_name(0);      # A
+    $str = xl_col_to_name(1);      # B
+    $str = xl_col_to_name(702);    # AAA
 
 
 The optional parameter C<$col_absolute> can be used to indicate if the column is absolute:
 
-    $str = xl_col_to_name(0, 0); # A
-    $str = xl_col_to_name(0, 1); # $A
-    $str = xl_col_to_name(1, 1); # $B
+    $str = xl_col_to_name( 0, 0 );    # A
+    $str = xl_col_to_name( 0, 1 );    # $A
+    $str = xl_col_to_name( 1, 1 );    # $B
 
 =head2 xl_range($row_1, $row_2, $col_1, $col_2, $row_abs_1, $row_abs_2, $col_abs_1, $col_abs_2)
 
@@ -592,12 +590,11 @@ The optional parameter C<$col_absolute> can be used to indicate if the column is
 
 This function converts zero based row and column cell references to an A1 style range string:
 
-    my $str = xl_range(0, 9, 0, 0); # A1:A10
-    my $str = xl_range(1, 8, 2, 2); # C2:C9
-
-    my $str = xl_range(0, 3, 0, 4      ); # A1:E4
-    my $str = xl_range(0, 3, 0, 4, 1   ); # A$1:E4
-    my $str = xl_range(0, 3, 0, 4, 1, 1); # A$1:E$4
+    my $str = xl_range( 0, 9, 0, 0 );          # A1:A10
+    my $str = xl_range( 1, 8, 2, 2 );          # C2:C9
+    my $str = xl_range( 0, 3, 0, 4 );          # A1:E4
+    my $str = xl_range( 0, 3, 0, 4, 1 );       # A$1:E4
+    my $str = xl_range( 0, 3, 0, 4, 1, 1 );    # A$1:E$4
 
 =head2 xl_range_formula($sheetname, $row_1, $row_2, $col_1, $col_2)
 
@@ -611,22 +608,22 @@ This function converts zero based row and column cell references to an A1 style
 
 This function converts zero based row and column cell references to an A1 style formula string:
 
-    my $str = xl_range_formula('Sheet1',   0,  9, 0, 0); # =Sheet1!$A$1:$A$10
-    my $str = xl_range_formula('Sheet2',   6, 65, 1, 1); # =Sheet2!$B$7:$B$66
-    my $str = xl_range_formula('New data', 1,  8, 2, 2); # ='New data'!$C$2:$C$9
+    my $str = xl_range_formula( 'Sheet1', 0, 9,  0, 0 ); # =Sheet1!$A$1:$A$10
+    my $str = xl_range_formula( 'Sheet2', 6, 65, 1, 1 ); # =Sheet2!$B$7:$B$66
+    my $str = xl_range_formula( 'New data', 1, 8, 2, 2 );# ='New data'!$C$2:$C$9
 
 This is useful for setting ranges in Chart objects:
 
     $chart->add_series(
-        categories    => xl_range_formula('Sheet1', 1, 9, 0, 0),
-        values        => xl_range_formula('Sheet1', 1, 9, 1, 1),
+        categories => xl_range_formula( 'Sheet1', 1, 9, 0, 0 ),
+        values     => xl_range_formula( 'Sheet1', 1, 9, 1, 1 ),
     );
 
     # Which is the same as:
 
     $chart->add_series(
-        categories    => '=Sheet1!$A$2:$A$10',
-        values        => '=Sheet1!$B$2:$B$10',
+        categories => '=Sheet1!$A$2:$A$10',
+        values     => '=Sheet1!$B$2:$B$10',
     );
 
 =head2 xl_inc_row($string)
@@ -638,10 +635,10 @@ This is useful for setting ranges in Chart objects:
 
 This functions takes a cell reference string in A1 notation and increments the row. The function will also handle Excel's absolute, C<$>, cell notation:
 
-    my $str = xl_inc_row('A1'  ); # A2
-    my $str = xl_inc_row('B$2' ); # B$3
-    my $str = xl_inc_row('$C3' ); # $C4
-    my $str = xl_inc_row('$D$4'); # $D$5
+    my $str = xl_inc_row( 'A1' );      # A2
+    my $str = xl_inc_row( 'B$2' );     # B$3
+    my $str = xl_inc_row( '$C3' );     # $C4
+    my $str = xl_inc_row( '$D$4' );    # $D$5
 
 =head2 xl_dec_row($string)
 
@@ -652,10 +649,10 @@ This functions takes a cell reference string in A1 notation and increments the r
 
 This functions takes a cell reference string in A1 notation and decrements the row. The function will also handle Excel's absolute, C<$>, cell notation:
 
-    my $str = xl_dec_row('A2'  ); # A1
-    my $str = xl_dec_row('B$3' ); # B$2
-    my $str = xl_dec_row('$C4' ); # $C3
-    my $str = xl_dec_row('$D$5'); # $D$4
+    my $str = xl_dec_row( 'A2' );      # A1
+    my $str = xl_dec_row( 'B$3' );     # B$2
+    my $str = xl_dec_row( '$C4' );     # $C3
+    my $str = xl_dec_row( '$D$5' );    # $D$4
 
 =head2 xl_inc_col($string)
 
@@ -666,10 +663,10 @@ This functions takes a cell reference string in A1 notation and decrements the r
 
 This functions takes a cell reference string in A1 notation and increments the column. The function will also handle Excel's absolute, C<$>, cell notation:
 
-    my $str = xl_inc_col('A1'  ); # B1
-    my $str = xl_inc_col('Z1'  ); # AA1
-    my $str = xl_inc_col('$B1' ); # $C1
-    my $str = xl_inc_col('$D$5'); # $E$5
+    my $str = xl_inc_col( 'A1' );      # B1
+    my $str = xl_inc_col( 'Z1' );      # AA1
+    my $str = xl_inc_col( '$B1' );     # $C1
+    my $str = xl_inc_col( '$D$5' );    # $E$5
 
 =head2 xl_dec_col($string)
 
@@ -679,36 +676,28 @@ This functions takes a cell reference string in A1 notation and increments the c
 
 This functions takes a cell reference string in A1 notation and decrements the column. The function will also handle Excel's absolute, C<$>, cell notation:
 
-    my $str = xl_dec_col('B1'  ); # A1
-    my $str = xl_dec_col('AA1' ); # Z1
-    my $str = xl_dec_col('$C1' ); # $B1
-    my $str = xl_dec_col('$E$5'); # $D$5
+    my $str = xl_dec_col( 'B1' );      # A1
+    my $str = xl_dec_col( 'AA1' );     # Z1
+    my $str = xl_dec_col( '$C1' );     # $B1
+    my $str = xl_dec_col( '$E$5' );    # $D$5
 
 =head1 TIME AND DATE FUNCTIONS
 
-
 Dates and times in Excel are represented by real numbers, for example "Jan 1 2001 12:30 AM" is represented by the number 36892.521.
 
 The integer part of the number stores the number of days since the epoch and the fractional part stores the percentage of the day in seconds.
 
-The epoch can be either 1900 or 1904. Excel for Windows uses 1900 and Excel for Macintosh uses 1904. The epochs are:
+A date or time in Excel is like any other number. To display the number as a date you must apply a number format to it: Refer to the C<set_num_format()> method in the Excel::Writer::XLSX documentation:
 
-    1900: 0 January 1900 i.e. 31 December 1899
-    1904: 1 January 1904
+    $date = xl_date_list( 2001, 1, 1, 12, 30 );
+    $format->set_num_format( 'mmm d yyyy hh:mm AM/PM' );
+    $worksheet->write( 'A1', $date, $format );    # Jan 1 2001 12:30 AM
 
-Excel on Windows and the Macintosh will convert automatically between one system and the other. By default Excel::Writer::XLSX uses the 1900 format. To use the 1904 epoch you must use the C<set_1904()> workbook method, see the Excel::Writer::XLSX documentation.
+The date handling functions below are supplied for historical reasons. In the current version of the module it is easier to just use the C<write_date_time()> function to write dates or times. See the DATES AND TIME IN EXCEL section of the main L<Excel::Writer::XLSX> documentation for details.
 
-There are two things to note about the 1900 date format. The first is that the epoch starts on 0 January 1900. The second is that the year 1900 is erroneously but deliberately treated as a leap year. Therefore you must add an extra day to dates after 28 February 1900. The functions in the following section will deal with these issues automatically. The reason for this anomaly is explained at http://support.microsoft.com/support/kb/articles/Q181/3/70.asp
+In addition to using the functions below you must install the L<Date::Manip> and L<Date::Calc> modules. See L<REQUIREMENTS> and the individual requirements of each functions.
 
-Note, a date or time in Excel is like any other number. To display the number as a date you must apply a number format to it: Refer to the C<set_num_format()> method in the Excel::Writer::XLSX documentation:
-
-    $date = xl_date_list(2001, 1, 1, 12, 30);
-    $format->set_num_format('mmm d yyyy hh:mm AM/PM');
-    $worksheet->write('A1', $date , $format); # Jan 1 2001 12:30 AM
-
-To use these functions you must install the C<Date::Manip> and C<Date::Calc> modules. See L<REQUIREMENTS> and the individual requirements of each functions.
-
-See also the DateTime::Format::Excel module,http://search.cpan.org/search?dist=DateTime-Format-Excel which is part of the DateTime project and which deals specifically with converting dates and times to and from Excel's format.
+For a C<DateTime.pm> solution see the L<DateTime::Format::Excel> module.
 
 =head2 xl_date_list($years, $months, $days, $hours, $minutes, $seconds)
 
@@ -727,10 +716,10 @@ See also the DateTime::Format::Excel module,http://search.cpan.org/search?dist=D
 
 This function converts an array of data into a number that represents an Excel date. All of the parameters are optional except for C<$years>.
 
-    $date1 = xl_date_list(2002, 1, 2);              # 2 Jan 2002
-    $date2 = xl_date_list(2002, 1, 2, 12);          # 2 Jan 2002 12:00 pm
-    $date3 = xl_date_list(2002, 1, 2, 12, 30);      # 2 Jan 2002 12:30 pm
-    $date4 = xl_date_list(2002, 1, 2, 12, 30, 45);  # 2 Jan 2002 12:30:45 pm
+    $date1 = xl_date_list( 2002, 1, 2 );                # 2 Jan 2002
+    $date2 = xl_date_list( 2002, 1, 2, 12 );            # 2 Jan 2002 12:00 pm
+    $date3 = xl_date_list( 2002, 1, 2, 12, 30 );        # 2 Jan 2002 12:30 pm
+    $date4 = xl_date_list( 2002, 1, 2, 12, 30, 45 );    # 2 Jan 2002 12:30:45 pm
 
 This function can be used in conjunction with functions that parse date and time strings. In fact it is used in most of the following functions.
 
@@ -750,16 +739,16 @@ This function converts a time string into a number that represents an Excel time
     hh:mm:ss.ss [AM|PM]
 
 
-The meridian, AM or PM, is optional and case insensitive. A 24 hour time is assumed if the meridian is omitted
+The meridian, AM or PM, is optional and case insensitive. A 24 hour time is assumed if the meridian is omitted.
 
-    $time1 = xl_parse_time('12:18');
-    $time2 = xl_parse_time('12:18:14');
-    $time3 = xl_parse_time('12:18:14 AM');
-    $time4 = xl_parse_time('1:18:14 AM');
+    $time1 = xl_parse_time( '12:18' );
+    $time2 = xl_parse_time( '12:18:14' );
+    $time3 = xl_parse_time( '12:18:14 AM' );
+    $time4 = xl_parse_time( '1:18:14 AM' );
 
 Time in Excel is expressed as a fraction of the day in seconds. Therefore you can calculate an Excel time as follows:
 
-    $time = ($hours*3600 +$minutes*60 +$seconds)/(24*60*60);
+    $time = ( $hours * 3600 + $minutes * 60 + $seconds ) / ( 24 * 60 * 60 );
 
 =head2 xl_parse_date($string)
 
@@ -773,20 +762,20 @@ Time in Excel is expressed as a fraction of the day in seconds. Therefore you ca
 
 This function converts a date and time string into a number that represents an Excel date.
 
-The parsing is performed using the C<ParseDate()> function of the Date::Manip module. Refer to the Date::Manip documentation for further information about the date and time formats that can be parsed. In order to use this function you will probably have to initialise some Date::Manip variables via the C<xl_parse_date_init()> function, see below.
+The parsing is performed using the C<ParseDate()> function of the L<Date::Manip> module. Refer to the C<Date::Manip> documentation for further information about the date and time formats that can be parsed. In order to use this function you will probably have to initialise some C<Date::Manip> variables via the C<xl_parse_date_init()> function, see below.
 
-    xl_parse_date_init("TZ=GMT","DateFormat=non-US");
+    xl_parse_date_init( "TZ=GMT", "DateFormat=non-US" );
 
-    $date1 = xl_parse_date("11/7/97");
-    $date2 = xl_parse_date("Friday 11 July 1997");
-    $date3 = xl_parse_date("10:30 AM Friday 11 July 1997");
-    $date4 = xl_parse_date("Today");
-    $date5 = xl_parse_date("Yesterday");
+    $date1 = xl_parse_date( "11/7/97" );
+    $date2 = xl_parse_date( "Friday 11 July 1997" );
+    $date3 = xl_parse_date( "10:30 AM Friday 11 July 1997" );
+    $date4 = xl_parse_date( "Today" );
+    $date5 = xl_parse_date( "Yesterday" );
 
 Note, if you parse a string that represents a time but not a date this function will add the current date. If you want the time without the date you can do something like the following:
 
-    $time  = xl_parse_date("10:30 AM");
-    $time -= int($time);
+    $time  = xl_parse_date( "10:30 AM" );
+    $time -= int( $time );
 
 =head2 xl_parse_date_init("variable=value", ...)
 
@@ -797,15 +786,15 @@ Note, if you parse a string that represents a time but not a date this function
 
     Requires:   Date::Manip
 
-This function is used to initialise variables required by the Date::Manip module. You should call this function before calling C<xl_parse_date()>. It need only be called once.
+This function is used to initialise variables required by the L<Date::Manip> module. You should call this function before calling C<xl_parse_date()>. It need only be called once.
 
-This function is a thin wrapper for the C<Date::Manip::Date_Init()> function. You can use C<Date_Init()>  directly if you wish. Refer to the Date::Manip documentation for further information.
+This function is a thin wrapper for the C<Date::Manip::Date_Init()> function. You can use C<Date_Init()>  directly if you wish. Refer to the C<Date::Manip> documentation for further information.
 
-    xl_parse_date_init("TZ=MST","DateFormat=US");
-    $date1 = xl_parse_date("11/7/97");  # November 7th 1997
+    xl_parse_date_init( "TZ=MST", "DateFormat=US" );
+    $date1 = xl_parse_date( "11/7/97" );    # November 7th 1997
 
-    xl_parse_date_init("TZ=GMT","DateFormat=non-US");
-    $date1 = xl_parse_date("11/7/97");  # July 11th 1997
+    xl_parse_date_init( "TZ=GMT", "DateFormat=non-US" );
+    $date1 = xl_parse_date( "11/7/97" );    # July 11th 1997
 
 =head2 xl_decode_date_EU($string)
 
@@ -819,20 +808,20 @@ This function is a thin wrapper for the C<Date::Manip::Date_Init()> function. Yo
 
 This function converts a date and time string into a number that represents an Excel date.
 
-The date parsing is performed using the C<Decode_Date_EU()> function of the Date::Calc module. Refer to the Date::Calc for further information about the date formats that can be parsed. Also note the following from the Date::Calc documentation:
+The date parsing is performed using the C<Decode_Date_EU()> function of the L<Date::Calc> module. Refer to the C<Date::Calc> documentation for further information about the date formats that can be parsed. Also note the following from the C<Date::Calc> documentation:
 
-"If the year is given as one or two digits only (i.e., if the year is less than 100), it is mapped to the window 1970 -2069 as follows":
+"If the year is given as one or two digits only (i.e., if the year is less than 100), it is mapped to the window 1970 -2069 as follows:"
 
-     0 E<lt>= $year E<lt>  70  ==>  $year += 2000;
-    70 E<lt>= $year E<lt> 100  ==>  $year += 1900;
+     0 <= $year <  70  ==>  $year += 2000;
+    70 <= $year < 100  ==>  $year += 1900;
 
 The time portion of the string is parsed using the C<xl_parse_time()> function described above.
 
 Note: the EU in the function name means that a European date format is assumed if it is not clear from the string. See the first example below.
 
-    $date1 = xl_decode_date_EU("11/7/97"); #11 July 1997
-    $date2 = xl_decode_date_EU("Sat 12 Sept 1998");
-    $date3 = xl_decode_date_EU("4:30 AM Sat 12 Sept 1998");
+    $date1 = xl_decode_date_EU( "11/7/97" );                    #11 July 1997
+    $date2 = xl_decode_date_EU( "Sat 12 Sept 1998" );
+    $date3 = xl_decode_date_EU( "4:30 AM Sat 12 Sept 1998" );
 
 =head2 xl_decode_date_US($string)
 
@@ -846,9 +835,9 @@ Note: the EU in the function name means that a European date format is assumed i
 
 This function converts a date and time string into a number that represents an Excel date.
 
-The date parsing is performed using the C<Decode_Date_US()> function of the Date::Calc module. Refer to the Date::Calc for further information about the date formats that can be parsed. Also note the following from the Date::Calc documentation:
+The date parsing is performed using the C<Decode_Date_US()> function of the L<Date::Calc> module. Refer to the C<Date::Calc> documentation for further information about the date formats that can be parsed. Also note the following from the C<Date::Calc> documentation:
 
-"If the year is given as one or two digits only (i.e., if the year is less than 100), it is mapped to the window 1970 -2069 as follows":
+"If the year is given as one or two digits only (i.e., if the year is less than 100), it is mapped to the window 1970 -2069 as follows:"
 
      0 <= $year <  70  ==>  $year += 2000;
     70 <= $year < 100  ==>  $year += 1900;
@@ -857,9 +846,9 @@ The time portion of the string is parsed using the C<xl_parse_time()> function d
 
 Note: the US in the function name means that an American date format is assumed if it is not clear from the string. See the first example below.
 
-    $date1 = xl_decode_date_US("11/7/97"); # 7 November 1997
-    $date2 = xl_decode_date_US("12 Sept Saturday 1998");
-    $date3 = xl_decode_date_US("4:30 AM 12 Sept Sat 1998");
+    $date1 = xl_decode_date_US( "11/7/97" );                 # 7 November 1997
+    $date2 = xl_decode_date_US( "Sept 12 Saturday 1998" );
+    $date3 = xl_decode_date_US( "4:30 AM Sept 12 Sat 1998" );
 
 =head2 xl_date_1904($date)
 
@@ -872,16 +861,14 @@ Note: the US in the function name means that an American date format is assumed
 
 This function converts an Excel date based on the 1900 epoch into a date based on the 1904 epoch.
 
-
-    $date1 = xl_date_list(2002, 1, 13); # 13 Jan 2002, 1900 epoch
-    $date2 = xl_date_1904($date1);      # 13 Jan 2002, 1904 epoch
-
+    $date1 = xl_date_list( 2002, 1, 13 );    # 13 Jan 2002, 1900 epoch
+    $date2 = xl_date_1904( $date1 );         # 13 Jan 2002, 1904 epoch
 
 See also the C<set_1904()> workbook method in the L<Excel::Writer::XLSX> documentation.
 
 =head1 REQUIREMENTS
 
-The date and time functions require functions from the C<Date::Manip> and C<Date::Calc> modules. The required functions are "autoused" from these modules so that you do not have to install them unless you wish to use the date and time routines. Therefore it is possible to use the row and column functions without having C<Date::Manip> and C<Date::Calc> installed.
+The date and time functions require functions from the L<Date::Manip> and L<Date::Calc> modules. The required functions are "autoused" from these modules so that you do not have to install them unless you wish to use the date and time routines. Therefore it is possible to use the row and column functions without having C<Date::Manip> and C<Date::Calc> installed.
 
 For more information about "autousing" refer to the documentation on the C<autouse> pragma.
 
@@ -899,7 +886,7 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
- MM-MMXII, John McNamara.
+Copyright MM-MMXIIII, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
 
